from fan.rpc import RPC
from fan.service import Service


class Context:
    def __init__(self, discovery, service=None, parent=None, name=None):
        self.discovery = discovery
        if service:
            assert isinstance(service, Service), service
        self.service = service
        self.parent = parent

        if isinstance(parent, Context):
            parent_context = parent.span.context
        else:
            parent_context = parent
        self.span = discovery.tracer.start_span(child_of=parent_context,
                                                operation_name=name)

    def create_child_context(self, name=None):
        return Context(self.discovery, self.service, self, name)

    @property
    def rpc(self):
        return RPC(self.create_child_context())

    def pre_call(self):
        pass

    def post_call(self):
        self.span.finish()



from distutils.core import setup

setup(
    name='fan',
    packages=['fan'],
    version='0.1.0',
    description='microservices kit',
    author='cybergrind',
    author_email='cybergrind@gmail.com',
    keywords=['rpc', 'microservices'],
    classifiers=[],
)



import logging


class Caller:
    log = logging.getLogger('Caller')

    def __init__(self, context, name):
        self.context = context
        self.call_path = [name]

    def __getattr__(self, name):
        self.call_path.append(name)
        return self

    def __call__(self, *args, **kwargs):
        self.log.debug('Generate call: {} {} {}'.format(self.call_path, args, kwargs))
        endpoint = self.context.discovery.find_endpoint(tuple(self.call_path[:-1]))
        self.log.debug('RPCEndpoint: {}'.format(endpoint))
        if endpoint:
            handler = self.find_handler(endpoint)
            try:
                self.context.pre_call()
                return handler(self.context, *args, **kwargs)
            finally:
                self.context.post_call()

    def find_handler(self, ep):
        self.log.debug('EP: {}'.format(ep))
        return getattr(ep, self.call_path[-1])


class RPC:
    def __init__(self, context):
        self.context = context

    def __getattr__(self, name):
        return Caller(self.context, name)
